"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const chalk_1 = require("chalk");
const cli_framework_1 = require("@ionic/cli-framework");
const guards_1 = require("../guards");
class Command extends cli_framework_1.BaseCommand {
    constructor(namespace, env) {
        super(namespace);
        this.namespace = namespace;
        this.env = env;
    }
    execute(inputs, options, runinfo) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const config = yield this.env.config.load();
            if (guards_1.isCommandPreRun(this)) {
                yield this.preRun(inputs, options, runinfo);
            }
            try {
                yield this.validate(inputs);
            }
            catch (e) {
                if (!this.env.flags.interactive) {
                    this.env.log.warn(`Command ran non-interactively due to ${chalk_1.default.green('--no-interactive')} flag, CI being detected, or a config setting.`);
                }
                throw e;
            }
            const runPromise = this.run(inputs, options, runinfo);
            const telemetryPromise = (() => tslib_1.__awaiter(this, void 0, void 0, function* () {
                if (config.telemetry !== false) {
                    let cmdInputs = [];
                    const metadata = yield this.getMetadata();
                    if (metadata.name === 'login' || metadata.name === 'logout') {
                        yield runPromise;
                    }
                    else if (metadata.name === 'help') {
                        cmdInputs = inputs;
                    }
                    else {
                        cmdInputs = yield this.getCleanInputsForTelemetry(inputs, options);
                    }
                    const cmd = this;
                    const path = yield cli_framework_1.generateCommandPath(cmd);
                    yield this.env.telemetry.sendCommand(path.map(([p]) => p).join(' '), cmdInputs);
                }
            }))();
            yield Promise.all([runPromise, telemetryPromise]);
        });
    }
    getCleanInputsForTelemetry(inputs, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const initialOptions = { _: [] };
            const metadata = yield this.getMetadata();
            const filteredInputs = inputs.filter((input, i) => !metadata.inputs || (metadata.inputs[i] && !metadata.inputs[i].private));
            const filteredOptions = Object.keys(options)
                .filter(optionName => {
                const metadataOption = metadata.options && metadata.options.find(o => {
                    return o.name === optionName || (typeof o.aliases !== 'undefined' && o.aliases.includes(optionName));
                });
                if (metadataOption && metadataOption.aliases && metadataOption.aliases.includes(optionName)) {
                    return false; // exclude aliases
                }
                if (!metadataOption) {
                    return true; // include unknown options
                }
                if (metadataOption.private) {
                    return false; // exclude private options
                }
                if (typeof metadataOption.default !== 'undefined' && metadataOption.default === options[optionName]) {
                    return false; // exclude options that match their default value (means it wasn't supplied by user)
                }
                return true;
            })
                .reduce((allOptions, optionName) => {
                allOptions[optionName] = options[optionName];
                return allOptions;
            }, initialOptions);
            const optionInputs = cli_framework_1.unparseArgs(filteredOptions, { useDoubleQuotes: true });
            return filteredInputs.concat(optionInputs);
        });
    }
}
exports.Command = Command;
