"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const chalk_1 = require("chalk");
const semver = require("semver");
const ailments_1 = require("../../doctor/ailments");
const npm_1 = require("../../utils/npm");
function registerAilments(registry, deps) {
    registry.register(new IonicForAngularUpdateAvailable(deps));
    registry.register(new IonicForAngularMajorUpdateAvailable(deps));
    // TODO: @ionic/core update available
    // TODO: Angular CLI update available
}
exports.registerAilments = registerAilments;
class AngularAilment extends ailments_1.Ailment {
    constructor(deps) {
        super(deps);
    }
}
// abstract class AutomaticallyTreatableAngularAilment extends AutomaticallyTreatableAilment {
//   protected readonly project: AngularProject;
//   constructor(deps: AutomaticallyTreatableAngularAilmentDeps) {
//     super(deps);
//   }
// }
class IonicForAngularUpdateAvailable extends AngularAilment {
    constructor() {
        super(...arguments);
        this.id = 'ionic-for-angular-update-available';
    }
    getVersionPair() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (!this.currentVersion || !this.latestVersion) {
                this.currentVersion = yield this.project.getFrameworkVersion();
                this.latestVersion = yield npm_1.pkgLatestVersion('@ionic/angular');
            }
            if (!this.currentVersion || !this.latestVersion) {
                return ['0.0.0', '0.0.0'];
            }
            return [this.currentVersion, this.latestVersion];
        });
    }
    getMessage() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const [currentVersion, latestVersion] = yield this.getVersionPair();
            return (`Update available for ${chalk_1.default.bold('@ionic/angular')}.\n` +
                `An update is available for ${chalk_1.default.bold('@ionic/angular')} (${chalk_1.default.cyan(currentVersion)} => ${chalk_1.default.cyan(latestVersion)}).\n`).trim();
        });
    }
    detected() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const [currentVersion, latestVersion] = yield this.getVersionPair();
            const diff = semver.diff(currentVersion, latestVersion);
            return diff === 'minor' || diff === 'patch';
        });
    }
    getTreatmentSteps() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const config = yield this.config.load();
            const { npmClient } = config;
            const [, latestVersion] = yield this.getVersionPair();
            const args = yield npm_1.pkgManagerArgs({ npmClient, shell: this.shell }, { command: 'install', pkg: `@ionic/angular@${latestVersion ? latestVersion : 'latest'}` });
            return [
                { name: `Visit ${chalk_1.default.bold('https://github.com/ionic-team/ionic/releases')} for each upgrade's instructions` },
                { name: `If no instructions, run: ${chalk_1.default.green(args.join(' '))}` },
                { name: `Watch for npm warnings about peer dependencies--they may need manual updating` },
            ];
        });
    }
}
class IonicForAngularMajorUpdateAvailable extends AngularAilment {
    constructor() {
        super(...arguments);
        this.id = 'ionic-for-angular-major-update-available';
    }
    getVersionPair() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (!this.currentVersion || !this.latestVersion) {
                this.currentVersion = yield this.project.getFrameworkVersion();
                this.latestVersion = yield npm_1.pkgLatestVersion('@ionic/angular');
            }
            if (!this.currentVersion || !this.latestVersion) {
                return ['0.0.0', '0.0.0'];
            }
            return [this.currentVersion, this.latestVersion];
        });
    }
    getMessage() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const [currentVersion, latestVersion] = yield this.getVersionPair();
            return (`Major update available for ${chalk_1.default.bold('@ionic/angular')}.\n` +
                `A major update is available for ${chalk_1.default.bold('@ionic/angular')} (${chalk_1.default.cyan(currentVersion)} => ${chalk_1.default.cyan(latestVersion)}).\n`).trim();
        });
    }
    detected() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const [currentVersion, latestVersion] = yield this.getVersionPair();
            const diff = semver.diff(currentVersion, latestVersion);
            return diff === 'major';
        });
    }
    getTreatmentSteps() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return [
                { name: `Visit ${chalk_1.default.bold('https://blog.ionicframework.com')} and ${chalk_1.default.bold('https://github.com/ionic-team/ionic/releases')} for upgrade instructions` },
            ];
        });
    }
}
